# Implementation design

Pynxtools-em addresses current challenges with information representation in the field of electron microscopy through leading by examples. Specifically, the tool implements functionalities for ontology matching of instance data from the proprietary representation to an open format and standard - [NeXus and the NXem application definitions and respective concepts](https://fairmat-nfdi.github.io/nexus_definitions). The examples which pynxtools-em currently supports are for sure a compromise. This is clear, if we consider how large the potential space of all possible naming conventions and combinations of pieces of information is that one could and in which formatting one could share knowledge within the field of electron microscopy (even if we ignore ow technically such information can be serialized using datatypes with different precision, languages, or encoding constraints). 

Therefore, the following design patterns guide our implementation:

- We do not consider that our work is complete (from the perspective of the idea in mind that a user can expect to drag-and-drop arbitrary content).
- We consider ontology matching a team effort that can only be achieved with technology partners and scientists working together.
- We acknowledge the efforts and key contributions that went into the development of file format reading libraries and data analysis libraries of the electron microscopy community. We had to start somewhere and we did so with tools from the hyperspy ecosystem [rosettasccio](https://github.com/hyperspy/rosettasciio), [pyxem](https://github.com/pyxem/pyxem), and [kikuchipy](https://github.com/pyxem/kikuchipy). We would be happy to work together with representatives from the many other great software packages within the [community] https://github.com/FAIRmat-NFDI/AreaB-software-tools/blob/main/software-list-b1-electron-microscopy.md).
- Our work is open to suggestions by the electron microscopy community, always realizing that just being able to read from a specific file alone is not solving the challenge that pynxtools-em addresses.
- We choose specific tangible examples of (meta)data semantic mapping for specific methods that are used in electron microscopy.
- Current examples implemented explore along two routes:
- There are examples of parsing capabilities (like TIFF and PNG) which address rather technical aspects (e.g. how to read from such files and pick technology-partner-specific formatting of instance data). These examples must not be understood as that they are of now implemented such that parsing arbitrary examples with that specific mime type. There are two practical reasons for this: Limited manpower to implement all this. Limited availability of reliable documentation what specific file formats from technology partners encode. Both challenges can be solved: The first one with support from the community. The second one with support of technology partners.
- There are examples of parsing examples for a specific method - for now Electron Backscatter Diffraction (EBSD) and Transmission Kikuchi Diffraction (TKD) - for which we explored how a large number of different formats can be parsed and that parsing be made more general and robust than to support just one prototypic example file. Again, we do not parse everything but rather an exemplar subset that suffices to offer a comprehensive example how at least a summary of e.g. the orientation map and region-of-interest that was analyzed in that research can be harmonized. The reason for this selection has been to motivated by the fact to show that there is at all a benefit of normalizing comprehensive and technically deep representations of electron microscopy data.

# Purpose and aim of pynxtools-em
We would like to provide context to the purpose and aim of pynxtools-em. The software implements a suggestion how diverse (meta)data from the research field of electron microscopy can be parsed and normalized to enable users to compare data. The software maps instance data from different formatting and concepts on a proposal for a common information exchange and representation via NeXus. The software achieves this through a two-stepped process of parsing. Firstly, via reading data from technology partner or specific serialization and formatting. Secondly, via applying transformations (if required) to map on NeXus. One of the key motivation for the development of [pynxtools and its plugins](https://github.com/FAIRmat-NFDI/pynxtools) was to explore and show how pieces of information can be harmonized and matched to enable the development of data-centric software tools and services in research data management systems (RDMS). The key reason to place such code into plugins rather than the RDM source code itself is to promote reusability, to offer users a stronger modularizability, tailoring and customizability of the RDM, and avoid duplication of development efforts that typically come with having to maintain many instead of a few codes.

# Software tools in electron microscopy - a mixture of proprietary and open-source solutions
Typically, users work with proprietary software from technology partners and custom-written software (many of which nowadays have an open-source license). Proprietary software offers frequently a lower usage barriers for end users surplus specifically tailored
access to and capabilities of storing instrument-specific (meta)data via a user interface that is optimized for working efficiently with the instrument.
As a burden, proprietary software write often (not exclusively though) to proprietary serialization formats (file or database entries). These formats are composed from proprietary semantic concepts. The key challenge is that the content and meaning of these concepts is very often not documented publicly. Therefore, there is a frequent necessity for having to convert between formats. When such conversions are performed ad hoc, substantial contextual information can get lost or become disconnected which makes tracking of workflows in electron microscopy difficult.

Several proprietary software tools implement the execution of script-based analyses. This scripting is also a key signature of the many software tools with an open-source development mindset and license. These offer an increasingly competitive alternative to proprietary software tools in electron microscopy. The combination of open source code, customizability, and the rooting (or often only reason for their existence) in exploring cutting-edge prototyping of algorithms and ideas by the scientific community, has made script-based software (especially in the Python world) nowadays a reality in many electron microscopy labs. This justifies thoughts on how using such software aligns with the aims of the [FAIR principles of data stewardship](https://doi.org/10.1162/dint_r_00024).

Here, script-based analyses can be considered a benefit and a burden when it comes to FAIR principles: The flexibility of being able to script ones analysis is a clear benefit. It can be considered also as a burden because of the current state of how such workflow is documented. Often this includes only the scripts themselves alongside some data in some processed state surplus the (close to) or final figures that were generated with the publication. Hence, it is often just assumed that these scripts not only work for different versions of the execution environment (i.e. different Matlab version) but also that users can obtain the same results - provided they run the scripts again using the data if these are provided.

This can leave room for substantial interpretation and ambiguity as there is neither a community agreed-upon standard of information exchange nor a thorough documentation of the execution environment, and possibly a lack of a serialized summary of the research via files or database entries. The practical challenge is not that no output files to such script execution are shared but that these are shared with a large variety of formats many of which using ad hoc data schemas. This is a substantial burden from the perspective of ontology matching because pieces of information are encoded and named differently although they (to human experts) represent instances of similar or even exactly the same concepts. So far it demands the capabilities of members and often domain experts within the electron microscopy community to assure that data can safely be compared from a scientific point of view and it is this not yet realized mapping which technically limits interoperable knowledge exchange in electron microscopy.
