# How to map pieces of information to NeXus

Conceptually, mapping between representations of concepts and instance data is a key tasks in information science. The plugin pynxtools-em implements this specifically for the file and serialization formats used within the research field of electron microscopy. The plugin offloads this code from eventually other software where NeXus is used to avoid that every user and software developer has to make individual changes in their own tool.

Technically, instance data representing concepts within the realm of one ontology or file format are considered the source (`src`) and mapped on instance data representing a concept in NeXus as the target (`trg`). This map is only performed by pynxtools-em, if `src` and `trg` can be transformed such that `src` and `trg` are connected via an `same_as` relationship. A transformation can be as simple as that an instance `src` has an elementary data type (e.g. a string or single-precision floating point value) that is copied into an instance `trg` of the same data type. Often though such a mapping demands further normalization. One example is when `src` represents quantity such as a tilt angle with unit radiants but `trg` requires that value to be in degrees. In this case, the transformation is composed of a copy and a multiplication to read the value from `src`, convert from radiants to degrees, and return the value as `trg`.

Such transformations are configured via the respective files in the *config* directory of pynxtools-epm.
Upon parsing, it is the set of function in *mapping_functor.py* which implements the actual transformations by reading the configuration and returning the properly formatted and transformed target to sucessively fill the `template` dictionary variable. The name functor is used as mapping may demand to use not only a copy or data type conversion as it was explained before. It is this *template* variable from which core functions like *convert.py* of the pynxtools write the actual NeXus/HDF5 file. The latter tool is also referred to as the dataconverter of [pynxtools](https://github.com/FAIRmat-NFDI/pynxtools).
